use std::{ffi::OsStr, path::PathBuf};

use anyhow::{anyhow, Result};
use clap::Parser;
use reqwest::StatusCode;
use std::{fs::File, io};

#[derive(Parser, Debug)]
struct Args {
    #[arg(short, long)]
    url: String,

    #[arg(short, long)]
    path: String,

    #[arg(short, long, default_value = "")]
    key: String,
}

fn main() -> Result<()> {
    let args = Args::parse();

    let path = if args.path == "-" {
        let mut line = String::new();
        io::stdin().read_line(&mut line)?;
        line.trim().into()
    } else {
        args.path
    };

    let path = PathBuf::from(&path);

    let file = File::open(&path)?;

    let client = reqwest::blocking::Client::new();

    let res = client
        .post(args.url)
        .query(&[(
            "name".to_string(),
            &path
                .file_name()
                .and_then(OsStr::to_str)
                .unwrap_or_default()
                .to_string(),
        )])
        .query(&[("key".to_string(), &args.key)])
        .body(file)
        .send()?;

    match res.status() {
        StatusCode::OK => {
            println!("{}", res.text()?);
            Ok(())
        }
        StatusCode::FORBIDDEN => Err(anyhow!("failed: invalid upload key! (403)")),
        StatusCode::BAD_REQUEST => Err(anyhow!(
            "failed: invalid request! (400) make sure all required params are present"
        )),
        _ => Err(anyhow!("failed: non-success status code")),
    }
}
